/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qcombobox.h>
#include <qcursor.h>

#include "calendar.h"
#include "wpast.h"
#include "wfuture.h"
#include "calendar.h"
#include <iostream.h>


#define min(a,b) (a)<(b)?(a):(b)

extern CalendarSettings config;
extern bool debuging;

/*!
    \fn Calendar::~Calendar()
	Default destructor of main window. 
*/
Calendar::~Calendar()
{
	delete line;
	delete future;
	delete past;
	delete menu;
	if (settingsWidget) 
		delete settingsWidget;
}


#include "calendar.moc"


/*!
    \fn Calendar::Calendar(QWidget *parent, const char *name, int f)
	Constructor of application window. 
		- Sets properties to initial values, 
		- creates GUI objects
		- creates CalendarCardData objects
		- create CalendarCardWidget objects
		- loads IndividualEvents.xml
		- loads PeriodicEvents.xml 
		- loads past.xml
	&nbsp;
		
	\param parent pointer to parent widget
	\param name name of widget
	\param f I don't know!
 */
	
Calendar::Calendar(QWidget *parent, const char *name, int f) 
 : QMainWindow(parent, name, f)
{
	//initialise variables
	//settings = ::config;
	future=NULL; 
	past=NULL;
	currentCard=NULL;
	settingsWidget=NULL;
    
	startTimer(60000);
    //initialise windows
	if ( !name )
		setName( tr("Calendar") );
    setEnabled( TRUE );
    setBackgroundOrigin( QWidget::WidgetOrigin );
    setFocusPolicy( QWidget::WheelFocus );
	

	//load settings
	this->loadSettings(parent->width(), parent->height());
	setPaletteBackgroundColor(::config.theme.backgroundApplicationColor);
	resize( QSize(::config.kalendarzWidth, ::config.HEIGHT+::config.MONTH_HEIGHT));

	
	initialiseCalendars();
	
	//creation of menu
	menu = new QPopupMenu(this);
	menuTitle = new QLabel(tr("Control panel"), menu);
	initialiseMenus();
}


/*!
    \fn Calendar::loadSettings(int screenWidth)
	Loads settings of the application from a file
	\param screenWidth physical width of a screen
 */
void Calendar::loadSettings(int screenWidth, int screenHeight)
{
    ::config.initialise(screenWidth, screenHeight);
}


/*!
    \fn Calendar::loadIndividualEvents()
	Loads individual events from a file and displays new events on the screen. 
 */
void Calendar::loadIndividualEvents()
{
	::config.file->loadIndividualEvents(&allIndividualEvents);
}


/*!
    \fn Calendar::loadPeriodicEvents()
 	Loads periodic events from a file and displays new events on the screen
 */
void Calendar::loadPeriodicEvents()
{
	::config.file->loadPeriodicEvents(&allPeriodicEvents);
}

/*!
    \fn Calendar::initialiseMenus()
	Creates popup menu items
 */
void Calendar::initialiseMenus()
{
	menu->insertItem(menuTitle);
	
	submenuAddPeriodic=new QPopupMenu(menu);
	addPeriodicSubmenuTextId=submenuAddPeriodic->insertItem(tr(""), this, SLOT(displaySettingsAddPeriodic()));
	
	addPeriodicSubmenuId=menu->insertItem(tr("Add Periodic"), submenuAddPeriodic);	
	menu->setItemVisible(addPeriodicSubmenuId, FALSE);
		
	submenuNavigation=new QPopupMenu(menu);
	submenuNavigation->setItemEnabled(submenuNavigation->insertItem(tr("One day forward")), FALSE);
	submenuNavigation->setItemEnabled(submenuNavigation->insertItem(tr("One day backward")), FALSE);
	submenuNavigation->setItemEnabled(submenuNavigation->insertItem(tr("One week forward")), FALSE);
	submenuNavigation->setItemEnabled(submenuNavigation->insertItem(tr("One week backward")), FALSE);
	submenuNavigation->setItemEnabled(submenuNavigation->insertItem(tr("One month forward")), FALSE);
	submenuNavigation->setItemEnabled(submenuNavigation->insertItem(tr("One month backward")), FALSE);
	submenuNavigation->setItemEnabled(submenuNavigation->insertItem(tr("One year forward")), FALSE);
	submenuNavigation->setItemEnabled(submenuNavigation->insertItem(tr("One year backward")), FALSE);
	submenuNavigation->setItemEnabled(submenuNavigation->insertItem(tr("Current date")), FALSE);
	menu->insertItem(tr("Navigation"), submenuNavigation);
	
	submenuSettings = new QPopupMenu(menu);

	submenuSettings->insertItem(tr("Add periodic event"), this, SLOT(displaySettings3()), CTRL+Key_N);
	submenuSettings->insertItem(tr("Remove periodic event"), this, SLOT(displaySettings2()), CTRL+Key_D);
	submenuSettings->insertItem(tr("Open configuration panel"), this, SLOT(displaySettings1()), CTRL+Key_O);
	submenuSettings->insertSeparator();
	submenuSettings->insertItem(tr("Reload application"), this, SLOT(reloadCalendars()));
	menu->insertItem(tr("Settings"), submenuSettings);

	menu->insertItem(tr("Quit"), this, SLOT(saveNClose()), CTRL+Key_Q);

}

/*!
    \fn Calendar::displaySettings(int i)
	Shows settings window.
	\param i number of tab which should be visible initialy
 */
void Calendar::displaySettings(int i)
{
 	if (!settingsWidget)
		{
		settingsWidget = new SettingsControl(this);	
		connect((QObject*)(settingsWidget->sliderCurrentWidth) , SIGNAL( valueChanged(int) ), this, SLOT( changeWidth(int) ));
		}
	
	settingsWidget->reloadThumbnails();
	
	//set active values of combos to real pixmaps	
	settingsWidget->comboDayBitmap->setCurrentItem(::config.getPixmapIndex(CalEnums::PXD));
	settingsWidget->comboIndivBitmap->setCurrentItem(::config.getPixmapIndex(CalEnums::PXI));
	settingsWidget->comboMonthBitmap->setCurrentItem(::config.getPixmapIndex(CalEnums::PXM));
	settingsWidget->comboNoEventBitmap->setCurrentItem(::config.getPixmapIndex(CalEnums::NO_PIXMAP));
	settingsWidget->comboWeekBitmap->setCurrentItem(::config.getPixmapIndex(CalEnums::PXW));
	settingsWidget->comboYearBitmap->setCurrentItem(::config.getPixmapIndex(CalEnums::PXY));	
	
	settingsWidget->show();
	settingsWidget->Settings->setCurrentPage(i-1);
}

/*!
    \fn Calendar::addIndividualEvent(QDate date, QString text)
	Adds individual events to the global list of individual events and saves a new file with individual events.
	\param date event date
	\param text event text
 */
 CalendarEvent * Calendar::addIndividualEvent(QDate date, QString text)
{
	CalendarEvent* temp=new CalendarEvent(date, text);
	this->allIndividualEvents.append(temp);
	
	::config.file->saveIndividualEvents(&(this->allIndividualEvents));
	future->addIndividualEvent(temp);
	return temp; 
}


/*!
    \fn Calendar::addPeriodicEvent(QDate dStart, QDate dEnd, QString t, CalEnums::event_type type, int period, bool* days, bool autodelete, int forgotafter)
	Adds periodic events into calendar and cause that new events will be displayed immediately
	\param dStart start of range date
	\param dEnd end of range date
	\param t text of event
	\param type type of event
	\param period period of event
	\param days table with indicators for weekdays
	\param autodelete indicates if event should be deleted if end date become less than current date
	\param forgotafter number of days event would be stored in past days calendar. 
	\return pointer to periodic event object
 */
CalendarEventPeriodic* Calendar::addPeriodicEvent(QDate dStart, QDate dEnd, QString t, CalEnums::event_type type, int period, bool* days, bool autodelete, int forgotafter)
{
	CalendarEventPeriodic * temp=new CalendarEventPeriodic(t, dStart, dEnd, type, period, days, autodelete, forgotafter);
	allPeriodicEvents.append(temp);
	
	if(dEnd>=QDate::currentDate())
		future->addPeriodicEvent(temp);	
	
	return temp;
}


/*!
    \fn Calendar::removeIndividualEvent(CalendarEvent* event)
	Removes individual event from a global list of individual events and saves file of individual events
	\param event individual event which has to be removed
 */
void Calendar::removeIndividualEvent(CalendarEvent* event)
{
	allIndividualEvents.removeRef(event);
	::config.file->saveIndividualEvents(&(this->allIndividualEvents));
}

/*!
    \fn Calendar::removePastEvent(CalendarEventPast* event)
	Removes individual event from a global list of individual events and saves file of periodical events
	\param event event which has to be removed
 */
void Calendar::removePastEvent(CalendarEventPast* event)
{
	allPastEvents.removeRef(event);
	::config.file->savePastEvents(&(this->allPastEvents));
}



/*!
    \fn Calendar::removePeriodicEvent(int i)
	Removes periodic event from a global list of periodic events and from list of periodic events of calendar of future events
	\param i index of event which has to be removed
 */
void Calendar::removePeriodicEvent(int i)
{
	
	if(future)
		future->removePeriodicEvent(allPeriodicEvents.at(i));
	allPeriodicEvents.remove(i);
}



/*!
    \fn Calendar::savePeriodicEvents()
	Slot to save periodic events. 
 */
void Calendar::savePeriodicEvents()
{
    ::config.file->savePeriodicEvents(&(this->allPeriodicEvents));
}


/*!
    \fn Calendar::reloadPeriodicEvents()
	Method reloads periodic event from file. Changes done in settings window are lost. 
 */
void Calendar::reloadPeriodicEvents()
{
	this->allPeriodicEvents.clear();
    this->loadPeriodicEvents();
	future->loadPeriodicEvents(&allPeriodicEvents);
}

/*!
    \fn Calendar::mousePressEvent ( QMouseEvent * e )
	Overloaded method QWidget::mousePressEvent to open popup menu
	\param e system event object
 */
void Calendar::mousePressEvent ( QMouseEvent * e )
{
	if(e->button() == Qt::RightButton)
		menu->popup(QWidget::mapToGlobal(e->pos()));
	if(e->button() == Qt::LeftButton && e->x()>(int)::config.lineposition-4 and e->x()<((int)::config.lineposition+4))
		{
		mouse_position=e->globalX();
		leftButtonPressed=TRUE;
		mouse_moved=FALSE;
		line->setFrameShadow(QFrame::Plain);
		}
}


/*!
    \fn Calendar::getPeriodicEvent(int i)
	Gets periodic event object of the given index from the global list of periodic events.
	\param i index on the global list of periodical events
	\return Periodical event object
 */
CalendarEventPeriodic* Calendar::getPeriodicEvent(int i)
{
   if(i<(signed long)(this->allPeriodicEvents.count()))
		return allPeriodicEvents.at(i);
	else 
		return NULL;
}

/*!
	\fn Calendar::resetCalendar()
	Method resets future and past calendars.
*/

void Calendar::resetCalendar()
{
	future->resetCalendar();
	past->resetCalendar();
}

/*!
	\fn Calendar::mouseReleaseEvent(QMouseEvent *e)
	Overloaded member of QWidget::mouseReleaseEvent.<br> Sets dividing line into sunken mode
	\param e event object
*/

void Calendar::mouseReleaseEvent(QMouseEvent * e)
{
	leftButtonPressed=FALSE;
	mouse_moved=FALSE;
	line->setFrameShadow(QFrame::Sunken);
}


/*!
    \fn Calendar::mouseMoveEvent(QMouseEvent * e)
	Overloaded member of QWidget::mouseMoveEvent<br>
	Moves dividing line following a mouse cursor
	\param e system event object
 */
void Calendar::mouseMoveEvent(QMouseEvent * e)
{
	if (leftButtonPressed && (mouse_moved || abs(e->globalX()-mouse_position)>5))
		{
			mouse_moved=TRUE;
			moveLine(e->globalX()-mouse_position);
			mouse_position=e->globalX();
		}	
	QWidget::mouseMoveEvent(e);
}



/*!
    \fn Calendar::moveLine(int delta)
	Moves dividing line by delta points. Method used when user drags a dividing line. 
	\param delta number of point dividing line is moved by
 */
void Calendar::moveLine(int delta)
{
	::config.moveLine(delta, past->max_zero_position+3);
	

	future->changeGeometry(::config.lineposition+6, ::config.kalendarzWidth);
	past->changeGeometry(0,::config.lineposition-3);
	line->move (::config.lineposition, 0);
	past->shift(0);
}


/*!
    \fn Calendar::changeWidth(int i)
	Change width of main application window and it resizes accordingly all application window elements.
	\param i new width of application window 
 */
void Calendar::changeWidth(int i)
{
	this->resize(i*::config.screenWidth/100, ::config.HEIGHT+::config.MONTH_HEIGHT);
	moveLineProportionaly(i*::config.screenWidth/100);
}


/*!
    \fn Calendar::moveLineProportionaly(int width)
	Moves dividing line basing on width of calendar. Method used when user changes width of application window.
	\param width new width of calendar. 
 */
void Calendar::moveLineProportionaly(int width)
{
	//move line proportionaly
	int newposition=width*::config.lineposition/(::config.kalendarzWidth);
	if (newposition > (int)(::config.nMaxPastCards*::config.WIDTH+4))
		newposition = ::config.nMaxPastCards*::config.WIDTH+4;
	if (newposition > past->max_zero_position +3)
		newposition = past->max_zero_position +3;

	line->move(newposition, 0);
	future->changeGeometry(newposition+4, width);
	past->changeGeometry(0, newposition-3);
	past->shift(0);

}


/*!
    \fn Calendar::loadPastEvents()
	Loads past events from a file and scans existent periodic and individual for past events.
 */
bool Calendar::loadPastEvents()
{
	bool save=FALSE;
	QDate savingDate;
	QDate tempDate;
	CalendarEventPeriodic *tempP;
	CalendarEvent *tempI;
	
	savingDate=::config.file->loadPastEvents(&allPastEvents);
	
	//for each day between saving date and current date check all periodic and individual events
	for(uint i=0;i<allPeriodicEvents.count();i++)
	{
		tempP=allPeriodicEvents.at(i);
		for(tempDate=savingDate;tempDate<QDate::currentDate();tempDate=tempDate.addDays(1))
		{
			if(tempP->doesMatch(tempDate))
			{
				addPastEvent(tempDate, tempP->text, tempP->maxDaysPassedForPeriodic);
				save=TRUE;	
			}
		}    
		if(tempP->endDate < QDate::currentDate() && tempP->deleteFromFileAfterEnd)
		{
			removePeriodicEvent(i);
			save=TRUE;	
			i--;
		}
	}
	for(uint i=0;i<this->allIndividualEvents.count();i++)
	{
		tempI=allIndividualEvents.at(i);
		for(tempDate=savingDate;tempDate<QDate::currentDate();tempDate=tempDate.addDays(1))
		{
			if(tempI->doesMatch(tempDate))
			{
				addPastEvent(tempDate, tempI->text, ::config.maxDaysPastForIndividuals);
				removeIndividualEvent(tempI);
				i--; //index was decreased because one event was deleted
				save=TRUE;
				break;
			}
		}    
	}

	return save;
}


/*!
    \fn Calendar::savePastEvents()
	Saves past events file.
 */
void Calendar::savePastEvents()
{
    ::config.file->savePastEvents(&(this->allPastEvents));
}


/*!
    \fn Calendar::saveIndividualEvents()
	Saves individual events file
 */
void Calendar::saveIndividualEvents()
{
    ::config.file->saveIndividualEvents(&(this->allIndividualEvents));
}



/*!
    \fn Calendar::addPastEvent(QDate date, QString string, int noOfDays)
	Adds past event to past events' calendar. If any event exists on the given date new event is merged with old one and time to life is calculated to max of times to life of both events.
	\param date date of event
	\param string description of event
	\param noOfDays time to life of event.
 */
void Calendar::addPastEvent(QDate date, QString string, int noOfDays)
{
	if(date.daysTo(QDate::currentDate())> noOfDays) return;
	CalendarEventPast *tempnew=new CalendarEventPast(date, string, noOfDays);
 	CalendarEventPast *temp;
	
	for (uint i=0; i<allPastEvents.count(); i++)
	{
		temp=allPastEvents.at(i);
		if(date<temp->startDate)
			{
			allPastEvents.insert(i, tempnew);
			return;
			}
		else if(date==temp->startDate)
			{
			temp->text+="\n"+tempnew->text;
			temp->timeToLife=(noOfDays>temp->timeToLife)?(noOfDays):(temp->timeToLife);
			return;
			}
	}
	this->allPastEvents.append(tempnew);	
}

/*!
    \fn Calendar::changeCurrentDate(QDate newDate)
	Changes current date of application - it may cause changes in calendars (adding or removing cards and moving between calendars.
	\param newDate new date
 */
void Calendar::changeCurrentDate(QDate newDate)
{
    ::config.current_date=newDate;
}


/*!
    \fn Calendar::removeOldCards(QDate newDate)
	Removes obsolete cards from future calendar. <br>
	Method does as following: <br>
	<ul><li>changes current date of application
	<li>moves all obsolete events (periodic and individual) to pas calendar
	<li>removes obsolete CalendarCardData objects
	<li>shits future calendar if CalendarCardData objects are linked to CalendarCardWidget objects. <br>
	<li>adds new cards for newly added events
	<li>moves dividing line if new past event is created. 
	</ul>
	<i>obsolete</i> means - older then current date of application.
	\param newDate new date of application.
 */

void Calendar::removeOldCards(QDate newDate)
{
	int nPastEvents;

	changeCurrentDate(newDate);
	//remember the number of past events
	nPastEvents = allPastEvents.count();
	destroyCalendars();
	initialiseCalendars();
	if (nPastEvents == 0 && allPastEvents.count()>0)
		moveLine(::config.WIDTH);
		
}


/*!
    \fn Calendar::initialiseCalendars()
	Creates past and future calendar. Loads data from files, creates everything
 */
void Calendar::initialiseCalendars()
{
	//loading settings from XML files.
	this->loadIndividualEvents();
	this->loadPeriodicEvents();
	if(this->loadPastEvents()) 
	{
		this->savePastEvents();
		this->saveIndividualEvents();
		this->savePeriodicEvents();
	}

	if (::debug)
	{
		
		cout << "---Events loaded from files---\nIndividual:  " << allIndividualEvents.count();
		cout << "\nPeriodcal:   " << allPeriodicEvents.count();
		cout << "\nPast:        " << allPastEvents.count() << "\n--- ---\n\n";
	}
	
	
		
	//create widget objects	
	future = new CalendarWidgetFuture((QWidget*)this, "Future Calendar", 0);
	past = new CalendarWidgetPast((QWidget*)this, "Past Calendar", 0, ::config.lineposition-4, this->allPastEvents.count());
	future->setFocusPolicy(QWidget::WheelFocus);
	past->setFocusPolicy(QWidget::WheelFocus);
	
	
	::config.divider=::config.lineposition;
	
	future->pos=::config.divider+4;
	future->width=::config.kalendarzWidth-future->pos;
	future->setGeometry(future->pos, 0, future->width, ::config.HEIGHT+::config.MONTH_HEIGHT);
	
	past->pos=0;
	past->width=::config.divider-4;
	past->zero_position=past->width;
	past->setGeometry(past->pos, 0, past->width, ::config.HEIGHT+::config.MONTH_HEIGHT);
	
	line = new QFrame( this, "line" );
	line->setGeometry( QRect (::config.lineposition, 0, 5,::config.HEIGHT+::config.MONTH_HEIGHT ) );
	line->setFrameShape( QFrame::VLine );
	line->setLineWidth( 2 );
	line->setFrameShadow(QFrame::Sunken);
	line->setCursor(QCursor(Qt::SplitHCursor));
	
	line->setPaletteBackgroundColor(::config.theme.applicationFontColor);
	
	//setting of some default values for settings
	
	future->loadIndividualEvents(&allIndividualEvents);
	future->loadPeriodicEvents(&allPeriodicEvents);
	past->loadPastEvents(&allPastEvents);

	if (::config.lineposition > ::config.WIDTH*allPastEvents.count()+3)
		moveLine(::config.WIDTH*allPastEvents.count()+3 - ::config.lineposition);
	
	future->show();
	past->show();
	line->show();
	

	mouse_moved=FALSE;
	leftButtonPressed=FALSE;
	mouse_position=0;
}


/*!
    \fn Calendar::destroyCalendars()
	Destroys past and future calendars. Used when current date is changed. 
 */
void Calendar::destroyCalendars()
{
	delete line;
	delete past;
	delete future;
	allIndividualEvents.clear();
	allPastEvents.clear();
	allPeriodicEvents.clear();
}


/*!
    \fn Calendar::timerEvent(QTimerEvent * e)
	Periodicaly checks if date was changed.
	In case it was - resets calendar application
 */
void Calendar::timerEvent(QTimerEvent * e)
{
    if (::config.current_date!=QDate::currentDate())
		removeOldCards(QDate::currentDate());

}

/*!
    \fn Calendar::reloadCalendars()
	Reloads application. Run from the popup menu and after changes in configuration are saved.
 */
void Calendar::reloadCalendars()
{
	::config.reloadTheme();
	if (settingsWidget)
		settingsWidget->reloadThumbnails();
	resizeCalendar();	
    removeOldCards(QDate::currentDate());
}


/*!
    \fn Calendar::changeAddPeriodicMenu(CalendarCardWidget* card)
	Method is used to add possibility of creation of new periodic event based on individual events. If an event doesn't exist yet it try to create periodic event with empty text and date of card. 
	@param card pointer to card which mouse button was clicked on
 */
void Calendar::changeAddPeriodicMenu(CalendarCardWidget* card)
{
	QString text=tr("Empty");
	if (card->pControl->IndividualEvent)
	{
	    text=card->pControl->IndividualEvent->text;
	}		
		
	menu->setItemVisible(addPeriodicSubmenuId, TRUE);
	submenuAddPeriodic->changeItem(addPeriodicSubmenuTextId, text);
	
	currentCard=card->pControl;
}


/*!
    \fn Calendar::displaySettingsAddPeriodic()
	Opens the settings tab with partialy filled data in case user choose "Add Periodic" field in menu using card with individual event. 
*/
void Calendar::displaySettingsAddPeriodic()
{
    displaySettings(3);
	if (currentCard)
		if(currentCard->IndividualEvent)
			settingsWidget->setPeriodic(currentCard->IndividualEvent->text, currentCard->date);
		else
			settingsWidget->setPeriodic("", currentCard->date);
	currentCard=NULL;
}



/*!
    \fn Calendar::resizeCalendar()
	Method resizes calendar and sets some background and font colors. Useful when theme was changed.
 */
void Calendar::resizeCalendar()
{
	line->setPaletteBackgroundColor(::config.theme.applicationFontColor);
	setPaletteBackgroundColor(::config.theme.backgroundApplicationColor);
	resize( QSize(::config.kalendarzWidth, ::config.HEIGHT+::config.MONTH_HEIGHT));
}


/*!
    \fn Calendar::saveNClose()
 */
void Calendar::saveNClose()
{
   ::config.setPos(this->pos());
   ::config.saveSettings();
   this->close();
}
