/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef CALENDAR_H
#define CALENDAR_H

#include <qmainwindow.h>
#include <qlabel.h>
#include <qpopupmenu.h>
#include <qptrlist.h>
#include <qdatetimeedit.h>
#include <qframe.h>
#include <qtimer.h>

#include "eperiod.h"
#include "settings.h"
#include "settingscontrol.h"
#include "InfoWidget.ui.h"

class CalendarWidget;
class CalendarWidgetPast;
class CalendarWidgetFuture;
class CalendarCardData;
class CalendarCardWidget;

/**
Main window of aplication. 
It manages events, settings, files, menus and is responsible for displaying calendars.

@author Adam Skowronski
*/
class Calendar : public QMainWindow
{
Q_OBJECT
public:

   /*! @name WIDGETS
   *  These variables are the objects of screen widgets
   *  @{
   */
	CalendarWidgetFuture *future; /**< calendar window with events in future displayed in */
	CalendarWidgetPast *past; /**< calendar window with past events displayed in */
    SettingsControl* settingsWidget;/**< window with settings of application */
	QFrame* line; /**< line dividing a main window between past and future calendar */
    /*! @} */
   
   /*! @name MENUS
   *  These variables are used to create a menu structure
   *  @{
   */
	QPopupMenu *menu; /**< popup menu window */
    QPopupMenu* submenuSettings;/**< submenu */
    QPopupMenu* submenuNavigation;/**< submenu */
    QPopupMenu *submenuAddPeriodic; /**< menu u	void timerEvent(QTimerEvent * e);
sed to easily add new periodic event */
    QLabel *menuTitle;/**< menu title label */
    QDateEdit* dateEdit;/**< date edit window displayed in menu */
    /*! @} */
	
	
	
   /*! @name Global variables
   *  These variables are used as a global variables. They are accessed not only by \class Calendar object
   *  @{
   */
    QPtrList<CalendarEvent> allIndividualEvents;/**< global list of individual events */
    QPtrList<CalendarEventPeriodic> allPeriodicEvents;/**< global list of periodical events */
    QPtrList<CalendarEventPast> allPastEvents;/**< global list of past events */
	/*! @} */

	/*! @name Constructor & Destructor
	* @{
	*/
    Calendar(QWidget *parent=0, const char *name=0, int f=0);
    ~Calendar();
	/*! @} */	

   /*! @name Event handlers
   * The following methods are called as a result of event happened in system. 
   *  @{
   */
    void mousePressEvent ( QMouseEvent * e );
    void mouseReleaseEvent ( QMouseEvent * e );
    void mouseMoveEvent ( QMouseEvent * e );
	void timerEvent(QTimerEvent * e);
	/*! @} */
	
	
	/*! @name Managing of calendar events
	* The following methods are used to manage events stored in Calendar application
	* @{
	*/
	CalendarEvent * addIndividualEvent(QDate d, QString t);
    CalendarEventPeriodic* addPeriodicEvent(QDate dStart, QDate dEnd, QString t, CalEnums::event_type type, int period=1, bool *days=NULL, bool autodelete=TRUE, int forgotafter=0);
    void 	addPastEvent(QDate date, QString string, int noOfDays);	
	CalendarEventPeriodic* 	getPeriodicEvent(int i);
    void 	removePeriodicEvent(int i);
    void 	removeIndividualEvent(CalendarEvent* event);
    void 	removePastEvent(CalendarEventPast* event);
	void 	loadIndividualEvents();
    void 	loadPeriodicEvents();
    bool 	loadPastEvents();
    void 	reloadPeriodicEvents();
    void 	savePastEvents();
    void 	saveIndividualEvents();
    /*! @} */
	
	/*! @name Managing menus
	* These methods are used to change popup menu and it's behavior 
	* @{
	*/	
	void initialiseMenus();
	void changeAddPeriodicMenu(CalendarCardWidget* card);
	/*! @} */

	
	void loadSettings(int screenWidth, int ScreenHeight);
	void resetCalendar();
	void displaySettings(int i);

	/*! @name Managing display
	* @{
	*/
    void moveLine(int delta);
    void moveLineProportionaly(int width);
	/*! @} */

	/*! @name Change date methods
	* These methods are mainly used during change of date.
	* @{
	*/	
	void changeCurrentDate(QDate newDate);
    void removeOldCards(QDate newDate);
	void initialiseCalendars();
	void destroyCalendars();
    void resizeCalendar();
    /*! @} */
   

	
public slots:	
	/** slot to run settings window with first tab active */
    void displaySettings1() {displaySettings(1);};
	/** slot to run settings window with second tab active */
    void displaySettings2() {displaySettings(2);};
	/** slot to run settings window with third tab active */
    void displaySettings3() {displaySettings(3);};
    /** \todo sprawdzic czy te metody s faktycznie slotami */
	void savePeriodicEvents();
    void changeWidth(int i);
    void reloadCalendars();
    void displaySettingsAddPeriodic();
    void saveNClose();


protected:
    bool leftButtonPressed;/**< indicates if left mouse button is presed */
    int mouse_position; /**< holds mouse position in which mouse button was pressed */
	bool mouse_moved; /**< indicates that mouse with button pressed was seriously moved */
    int addPeriodicSubmenuId;	/**< menu id of AddPeriodic submenu */
    int addPeriodicSubmenuTextId; /**< menu id of AddPeriodicSubmenu content */
    CalendarCardData* currentCard; /**< pointer to current, clicked calendar card */
};

#endif
