/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "findiv.h"
#include <qobject.h>
#include <qmessagebox.h>
#include "consts.h"

/*!
	\fn FileIndividual::FileIndividual(QPtrList<CalendarEvent>* l)
	Constructor of a class
	\param l pointer to global list of individual events
*/
FileIndividual::FileIndividual(QPtrList<CalendarEvent>* l)
{
	list=l;
}

/*!
	\fn FileIndividual::~FileIndividual()
	Default destructor
*/
FileIndividual::~FileIndividual()
{
}

/*!
    \fn FileIndividual::createIndividualXML()
	Methods creates xml string, which will be used to save in a file, using list of individual events
	\return String to save in a file
*/
QString FileIndividual::createIndividualXML()
{
	QString text_xml;
	//create doc with <IndividualEvents>
	text_xml+="<IndividualEvents>\n";
	//insert <head> section
	text_xml+="\t<head>\n";
		//insert <description>
	text_xml+="\t\t<description>";
	text_xml+=QObject::tr("File with individual events");
	text_xml+="</description>\n";
	text_xml+="\t</head>\n";
	//insert <data> section
	text_xml+="\t<data>\n";
		//for each periodic event
		for(int i=0; i<(signed long)(list->count()); i++)
			text_xml+=addTagEventIndividual(list->at(i));
	text_xml+="\t</data>\n";
	text_xml+="</IndividualEvents>";
	return text_xml;
}

/*!
    \fn FileIndividual::addTagEventIndividual(CalendarEvent * temp)
	Method creates XML string describing individual event
 	\param temp pointer to individual event
	\return XML string describing individual event
*/
QString FileIndividual::addTagEventIndividual(CalendarEvent * temp)
{
	QString text_xml="\t\t<event>\n";
	
	text_xml+=addTagAny("date",3,temp->startDate.toString("yyyyMMdd"));
	text_xml+=addTagAny("text",3,temp->text);
	
	text_xml+="\t\t</event>\n";	
	
	return text_xml;
}






/*!
    \fn FileIndividual::createIndividualList(QString filename)
	Method creates global list of individual events using xml configuration file.
 	\param filename name of file with individual events stored in.
 */
void FileIndividual::createIndividualList(QString filename)
{
	QDomElement doc;
	if(openXMLFileForRead(filename, "IndividualEvents", &doc))
		return;
	QDomNode n = doc.firstChild();
    while( !n.isNull() ) {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if( !e.isNull() ) {
			if (e.tagName()=="data")
				{
					QDomNode n1;
					n1=n.firstChild();
					retrieveIndividualEvents(n1);
				}
        }
        n = n.nextSibling();
    }
}


/*!
    \fn FileIndividual::retrieveIndividualEvents(QDomNode n)
	Method retrieves all individual events saved in individual events cofniguration file
 	\param n pointer to first &lt;event&gt; tag.
 */
void FileIndividual::retrieveIndividualEvents(QDomNode n)
{
 	//pobiera wszystkie eventy
	while (!n.isNull() )
	{
	QDomElement e = n.toElement();
	if( !e.isNull() )
		{
		if (e.tagName()=="event")
			{
				QDomNode n1;
				n1=n.firstChild();
				retrieveIndividualEventParameters(n1);
			}
		}
	n=n.nextSibling();
	}
}


/*!
    \fn FileIndividual::retrieveIndividualEventParameters(QDomNode n)
	Method retrieves data of individual event from xml configuration file
	\param n pointer to first data of individual event
 */
void FileIndividual::retrieveIndividualEventParameters(QDomNode n)
{

	QString text, temp, sError="", errorCaption="";
	QDate dStart;
	QDate bigdate(2999,12,31);
	int error=0;
	dStart=bigdate;
	
	while(!n.isNull() )
	{
		QDomElement e = n.toElement();
		if( !e.isNull() )
		{
			//cout << "\t\t" << e.tagName() << "\n";	
			temp=e.text();
			
			if(e.tagName()=="date")
			{
				if (temp.length()==8)
				{
					dStart.setYMD(atoi(temp.left(4)), atoi(temp.mid(4,2)), atoi(temp.mid(6,2)));
					//cout << "\t\t\t" << dStart.toString() << "\n";
				}
				else
				{
					error|=CalEnums::ERR_DSTART;
					sError+="date = "+temp+QObject::tr("\nShould be in YYYYMMDD format (ie. 20021231)\n");
				}
			}
			else if(e.tagName()=="text")
			{
				text=temp;
				//cout << "\t\t\t" << text << "\n";
				if(text.length()==0)
					{
					error|=CalEnums::ERR_GENERAL;
					sError+="text = "+temp+QObject::tr("\nShould be populated\n");
					}
			}
		}
	n=n.nextSibling();
	}

	if(!(error&CalEnums::ERR_DSTART) && dStart==bigdate)
		{
			error|=CalEnums::ERR_DSTART;
			sError+=QObject::tr("Individual event date has to be specified\n");
		}
	//sprawdzenie czy nie ma bdw
	if(error)
	{
		errorCaption+=QObject::tr("Check which event is error about and add it again.\nYou can also edit manualy file with periodic events if you wantJ\n(~/.kalendarz/individual.xml).");
		errorCaption+="\n_____________________________________\n";
		errorCaption+=QObject::tr("Error concerned the following event:");
		if(!(error & CalEnums::ERR_DSTART))
			errorCaption+="\ndate="+dStart.toString();
		if(!(error & CalEnums::ERR_GENERAL))
			errorCaption+="\ntekst="+text;
		errorCaption+=QString("%1").arg(error);
		errorCaption+="\n_____________________________________\n";
		errorCaption+=QObject::tr("Errors occured:\n")+sError;
		QMessageBox a(QObject::tr("Error in XML file of individual events"), errorCaption, QMessageBox::Warning,0,0,0);
		a.exec();
	}
	else
	{
		CalendarEvent * tempkIE=new CalendarEvent(dStart, text);
		list->append(tempkIE);
	}
}



