/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "settings.h"
#include "consts.h"
#include <iostream.h>
#include <qsettings.h>
#include <stdlib.h>


/* images */
#include "alert.xpm"
#include "normal.xpm"
#include "daily.xpm"
#include "weekly.xpm"
#include "monthly.xpm"
#include "yearly.xpm"
#include "individual.xpm"

extern bool debuging;

CalendarSettings::CalendarSettings()
{
	file=NULL;
}


CalendarSettings::~CalendarSettings()
{
	if (file)
		delete file;
	for (int i = 0; i<7; i++)
		{
			if (!pixmaps[i]->isNull())
				delete pixmaps[i];
		}
}


/*!
    \fn CalendarSettings::loadPixmaps()
	Initialy associates pixmaps with event types
 */
void CalendarSettings::loadPixmaps()
{
	//loading pixmaps from file. 
	//if there is no file for pixmap - the default pixmap is used
	QString theme_path=PATH+theme_directory+theme.name;
	
	pixmaps[0]=new QPixmap(theme_path+"/normal.xpm");
	if(pixmaps[0]->isNull()) 
		pixmaps[0]=new QPixmap((const char **) normal_xpm);
	pixmaps[1]=new QPixmap(theme_path+"/daily.xpm");
	if(pixmaps[1]->isNull())
	 	pixmaps[1]=new QPixmap((const char **) daily_xpm);
	pixmaps[2]=new QPixmap(theme_path+"/weekly.xpm");
	if(pixmaps[2]->isNull()) 
		pixmaps[2]=new QPixmap((const char **) weekly_xpm);
	pixmaps[3]=new QPixmap(theme_path+"/monthly.xpm");
	if(pixmaps[3]->isNull())
	 	pixmaps[3]=new QPixmap((const char **) monthly_xpm);
	pixmaps[4]=new QPixmap(theme_path+"/yearly.xpm");
	if(pixmaps[4]->isNull())
	 	pixmaps[4]=new QPixmap((const char **) yearly_xpm);
	pixmaps[5]=new QPixmap(theme_path+"/individual.xpm");
	if(pixmaps[5]->isNull()) 
		pixmaps[5]=new QPixmap((const char **) individual_xpm);
	pixmaps[6]=new QPixmap(theme_path+"/alert.xpm");
	if(pixmaps[6]->isNull())
	 	pixmaps[6]=new QPixmap((const char **) alert_xpm);

		
	//setting width and height of pixmaps. 
	int w=0;
	int h=0;
	for (int i=0; i<=6; i++)
		{
			if(w<pixmaps[i]->width()) w=pixmaps[i]->width();
			if(h<pixmaps[i]->height()) h=pixmaps[i]->height();
		}
	//set global parameters.
	WIDTH=w;
	HEIGHT=h;
	
}


/*!
    \fn CalendarSettings::setPixmap(int index, int pixmap_nr)
	Associates pixmap to type of event
	\param index index of type of event
	\param pixmap_nr index of pixmap assigned to this type of event
 */
void CalendarSettings::setPixmap(int index, int pixmap_nr)
{
	current_pixmaps[index]=pixmap_nr;
}

/*!
    \fn CalendarSettings::getPixmap(int index)
	\param index index of type of event
	\return Pixmap assigned to this type of event
 */
QPixmap* CalendarSettings::getPixmap(int index)
{
//	return &pixmaps[current_pixmaps[index]];
	return pixmaps[current_pixmaps[index]];
}







/*!
    \fn CalendarSettings::getPixmapIndex(int index)
	retrieves a number of pixmap for given index/state
	\param index index of pixmap in current_pixmaps table
	\sa current_pixmaps
 */
int CalendarSettings::getPixmapIndex(int index)
{
    return current_pixmaps[index];
}



/*!
    \fn void CalendarSettings::initialise(int sw)
	Initialise settings. Loads settings files, sets properties
	\param sw screen width in pixels
 */
void CalendarSettings::initialise(int sw, int sh)
{
	current_date=QDate::currentDate();
  	
	
	PATH=QString(getenv("HOME")) + QString("/.kalendarz/");
	
	//loading pixmaps from theme with the default order
	setPixmap(CalEnums::NO_PIXMAP, 0);
	setPixmap(CalEnums::PXD, 1);
	setPixmap(CalEnums::PXW, 2);
	setPixmap(CalEnums::PXM, 3);
	setPixmap(CalEnums::PXY, 4);
	setPixmap(CalEnums::PXI, 5);
	setPixmap(CalEnums::PXALERT, 6);

	/* loading settings file */
	file = new CalendarFile();

	//load settings from file
	loadSettings();
	
	validateSettings();
	
	/* loading pixmaps belonging to theme set in settings file */
	loadPixmaps();

	screenWidth=sw;
	screenHeight=sh;
	noOfKalWidgets=screenWidth/WIDTH+2;
	this->kalendarzWidth=screenWidth*currentCalendarWidth/100;
	if (nPosX > screenWidth - WIDTH || nPosX < 0)	
		nPosX = 0;
	if (nPosY > screenHeight - HEIGHT || nPosY < 0)
		nPosX = 0;
	
	/* settings the rest of theme parameters: fonts, colors */
	theme.setTheme(HEIGHT, PATH+"themes/"+theme.name);
	MONTH_HEIGHT=HEIGHT/3;

	if(debuging)
	{
		printDebugInformation();
	}
	
}


/*!
    \fn CalendarSettings::validateSettings()
	Validates settings read from file. If any of settings is invalid - changes it's value to default one.
 */
void CalendarSettings::validateSettings()
{
	/* set default setting before reading from settings file */
	if(currentCalendarWidth<10) currentCalendarWidth=10;
	else if(currentCalendarWidth>100) currentCalendarWidth=100;
	if (theme.name.isEmpty())
		theme.name="default";
	for (int i=0; i<6;i++)
		{
			if(current_pixmaps[i]<0)
				current_pixmaps[i]=0;
			else if (current_pixmaps[i]>6)
				current_pixmaps[i]=6;
		}
	if (nMaxPastCards<0)
		nMaxPastCards=10;
	if(this->maxDaysPastForIndividuals<0)
		this->maxDaysPastForIndividuals=100;
	divider=nMaxPastCards*WIDTH+3;
}


/*!
    \fn void CalendarSettings::moveLine(int delta, int max_value)
	resizes past and current calendar and moves dividing line
	\param delta how many points divider should be moved
	\param max_value maximum value of divider position
 */
void CalendarSettings::moveLine(int delta, int max_value)
{
    lineposition +=delta;
	if (lineposition < 0)
		lineposition = 0;
	if ((int)lineposition > kalendarzWidth)
		lineposition=kalendarzWidth;
	if ((int)lineposition > nMaxPastCards*WIDTH+4)
		lineposition = nMaxPastCards*WIDTH+4;
	if ((int)lineposition > max_value)
		lineposition = max_value;
}


/*!
    \fn CalendarSettings::loadSettings()
	Loads settings of application using QSettings object
 */
void CalendarSettings::loadSettings()
{
   	QSettings * set=new QSettings(QSettings::Ini);
	set->setPath("mike.oldfield.org.pl/tytus", "Kalendarz", QSettings::User);
	set->insertSearchPath(QSettings::Unix,this->PATH );
	this->theme.name=set->readEntry("/kalendarz_/Theme/THEME");
	this->current_pixmaps[0]=set->readNumEntry("/kalendarz_/Pixmap_order/P1",0);
	this->current_pixmaps[1]=set->readNumEntry("/kalendarz_/Pixmap_order/P2",1);
	this->current_pixmaps[2]=set->readNumEntry("/kalendarz_/Pixmap_order/P3",2);
	this->current_pixmaps[3]=set->readNumEntry("/kalendarz_/Pixmap_order/P4",3);
	this->current_pixmaps[4]=set->readNumEntry("/kalendarz_/Pixmap_order/P5",4);
	this->current_pixmaps[5]=set->readNumEntry("/kalendarz_/Pixmap_order/P6",5);
	this->current_pixmaps[6]=set->readNumEntry("/kalendarz_/Pixmap_order/P7",6);
	this->closeOtherInfoWindows=set->readBoolEntry("/kalendarz_/Info_behavior/CLOSE_ALL_INFOS", TRUE);
	this->closeInfoWindowAcceptItsContent=!(set->readBoolEntry("/kalendarz_/Info_behavior/OK_ACCEPTS_INFO",FALSE));
	this->bMarkSunday=set->readBoolEntry("/kalendarz_/Display/MARK_SUNDAY",TRUE);
	this->bMarkActive=set->readBoolEntry("/kalendarz_/Display/MARK_ACTIVE",TRUE);
	this->currentCalendarWidth=set->readNumEntry("/kalendarz_/CALENDAR_WIDTH",100);
	this->lineposition=set->readNumEntry("/kalendarz_/DIVIDER",10);
	this->nMaxPastCards=set->readNumEntry("/kalendarz_/MAX_PAST_CARDS",10);
	this->maxDaysPastForIndividuals=set->readNumEntry("/kalendarz_/DELETE_INDIVIDUAL_AFTER",0);
	this->nPosX=set->readNumEntry("/kalendarz_/POS_X",0);
	this->nPosY=set->readNumEntry("/kalendarz_/POS_Y",0);
	delete set;
}


/*!
    \fn CalendarSettings::saveSettings()
	Saves settings of application using QSettings object
 */
void CalendarSettings::saveSettings()
{
   	QSettings * set=new QSettings(QSettings::Ini);
	set->setPath("mike.oldfield.org.pl/tytus", "Kalendarz", QSettings::User);
	set->insertSearchPath( QSettings::Unix, this->PATH );
	set->writeEntry("/kalendarz_/Theme/THEME", this->theme.name);
	set->writeEntry("/kalendarz_/Pixmap_order/P1",this->getPixmapIndex(0));
	set->writeEntry("/kalendarz_/Pixmap_order/P2",this->getPixmapIndex(1));
	set->writeEntry("/kalendarz_/Pixmap_order/P3",this->getPixmapIndex(2));
	set->writeEntry("/kalendarz_/Pixmap_order/P4",this->getPixmapIndex(3));
	set->writeEntry("/kalendarz_/Pixmap_order/P5",this->getPixmapIndex(4));
	set->writeEntry("/kalendarz_/Pixmap_order/P6",this->getPixmapIndex(5));
	set->writeEntry("/kalendarz_/Pixmap_order/P7",this->getPixmapIndex(6));
	set->writeEntry("/kalendarz_/Info_behavior/CLOSE_ALL_INFOS",this->closeOtherInfoWindows);
	set->writeEntry("/kalendarz_/Info_behavior/OK_ACCEPTS_INFO",!this->closeInfoWindowAcceptItsContent);
	set->writeEntry("/kalendarz_/Display/MARK_SUNDAY",this->bMarkSunday);
	set->writeEntry("/kalendarz_/Display/MARK_ACTIVE",this->bMarkActive);
	set->writeEntry("/kalendarz_/POS_X", this->nPosX);
	set->writeEntry("/kalendarz_/POS_Y", this->nPosY);
	set->writeEntry("/kalendarz_/CALENDAR_WIDTH",this->currentCalendarWidth);
	set->writeEntry("/kalendarz_/DIVIDER",(int)this->lineposition);
	set->writeEntry("/kalendarz_/MAX_PAST_CARDS",this->nMaxPastCards);
	set->writeEntry("/kalendarz_/DELETE_INDIVIDUAL_AFTER",this->maxDaysPastForIndividuals);
	delete set;
}


/*!
    \fn CalendarSettings::reloadTheme()
	Reloads theme settings. Useful when theme is changing. 
 */
void CalendarSettings::reloadTheme()
{
	loadPixmaps();
	noOfKalWidgets=screenWidth/WIDTH+2;
	this->kalendarzWidth=screenWidth*currentCalendarWidth/100;
	theme.setTheme(HEIGHT, PATH+"themes/"+theme.name);
	MONTH_HEIGHT=HEIGHT/3;
 	if (debuging)
		printDebugInformation();
}


/*!
    \fn CalendarSettings::printDebugInformation()
	Method to write all configuration to standard output stream
 */
void CalendarSettings::printDebugInformation()
{
	cout <<   "---Settings values---\nbMarkActive                     " << bMarkActive;
	cout << "\nbMarkSunday                     " << bMarkSunday;
	cout << "\ncloseInfoWindowAcceptItsContent " << closeInfoWindowAcceptItsContent;
	cout << "\ncloseOtherInfoWindows           " << closeOtherInfoWindows;
	cout << "\ncurrentCalendarWidth            " << currentCalendarWidth;
	cout << "\ncurrent_date                    " << current_date.toString();
	cout << "\ndivider                         " << divider;
	cout << "\nfile                            " << file;
	cout << "\nkalendarzWidth                  " << kalendarzWidth;
	cout << "\nlineposition                    " << lineposition;
	cout << "\nmaxDaysPastForIndividuals       " << maxDaysPastForIndividuals;
	cout << "\nmaxPastCardOnScreen             " << maxPastCardOnScreen;
	cout << "\nnMaxPastCards                   " << nMaxPastCards;
	cout << "\nnoOfKalWidgets                  " << noOfKalWidgets;
	cout << "\nPATH                            " << PATH;
	cout << "\npixmaps                         " << pixmaps;
	cout << "\nscreenWidth                     " << screenWidth;
	cout << "\ntheme                           " << theme.name;
	cout << "\nMONTH_HEIGHT                    " << MONTH_HEIGHT;
	cout << "\nnPosX                           " << nPosX;
	cout << "\nnPosY                           " << nPosY;
	cout << "\nHEIGHT                          " << HEIGHT;
	cout << "\nWIDTH                           " << WIDTH << "\n---End of settings values---\n\n";
}


/*!
    \fn CalendarSettings::setPos(QPoint)
 */
void CalendarSettings::setPos(QPoint p)
{
    nPosX=p.x();
	nPosY=p.y();
}
