/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SETTINGS_H
#define SETTINGS_H

#include <qstring.h>
#include <qpixmap.h>
#include <qdatetime.h>

#include "theme.h"
#include "file.h"

class CalendarFile;
class CalendarTheme;


/**
This class is storing all global settings data

@brief Global settings class
@author Adam Skowronski
*/
class CalendarSettings{
public:
    CalendarSettings();

    ~CalendarSettings();

	/** \name Settings managing */
	/** @{ */
    void initialise(int sw, int sh);
    void validateSettings();
    void loadSettings();
    void saveSettings();
    void reloadTheme();
	/** @} */

	/** \name Pixmap managing */
	/** @{ */
    void loadPixmaps();
	void setPixmap(int index, int pixmap_nr);
    QPixmap* getPixmap(int index);
    int getPixmapIndex(int index);
	/** @} */

    void moveLine(int delta, int max_value);
    void printDebugInformation();
    void setPos(QPoint);


public:
	/** \name Card properties */
	/** @{ */
    int HEIGHT;					/**< height of calendar card. Calculated as a max(heights of theme bitmaps ) */
	int WIDTH;					/**< width of calendar card. Calculated as a max(widthgs of theme bitmaps) */
    int MONTH_HEIGHT;			/**< height of part of CalendarWidget where name of month is displayed. Curently equals HEIGHT/3 */
	/** @} */

	/** \name Files properties */
	/** @{ */
    QString PATH;				/**< path to file. should be equal to ~/.kalendarz */
    CalendarFile* file;			/**< pointer to CalendarFile object. Used for file operations */
	/** @} */

	/** \name Themes properties */
	/** @{ */
    CalendarTheme theme;		/**< theme object */
	QPixmap *pixmaps[7];		/**< array of pixmaps used in application */
	/** @} */

	/** \name Calendar behavior properties */
	/** @{ */
	bool closeOtherInfoWindows;	/**< indicates if opening of InfoWidget should cause closing any other InfoWidget window */
	bool closeInfoWindowAcceptItsContent;
								/**< indicates if values in InfoWidget are not accepted only if "Cancel" button is pressed.  */
    bool bMarkSunday;			/**< indicator if Sundays should be different than normal days. Differencies are stored in CalendarTheme object */
    bool bMarkActive;			/**< indicator if card should be changed if InfoWindow is open. */
    int maxDaysPastForIndividuals;
								/**< how long individual should be remembered in file */
	/** @} */

	/** \name Calendar properties */
	/** @{ */
	QDate current_date; 		/**< remembers current date. Used when date is changed */

	int current_pixmaps[7];		/**< asigment of pixmaps from pixmaps to type of events */
    int kalendarzWidth;			/**< width of application. Its combined of screenWidth and currentWidth parameter */
    int noOfKalWidgets;		/**< number of CalendarCardWidget object displayed on screen */ 
    int currentCalendarWidth;	/**< percent of screenwidth which should be used to display calendar application */
    int screenWidth;			/**< width of the screen - calculated during initialisation, so change screen resolution during work of application causes nothing */
    int maxPastCardOnScreen;	/**< how many cards can be used for past events */
	int nMaxPastCards; 			/**< maximal number of cards od past calendar displayed on screen */

	int divider; 				/**< saved position of vertical line dividing past and future calendar */
	uint lineposition; 			/**< currend position of vertical line dividing past and future calendar - used during moving of divider */
	/** @} */
    int nPosX;
    int nPosY;
    int screenHeight;
};

#endif
