/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qcombobox.h>
#include <qdatetimeedit.h>
#include <qspinbox.h>
#include <qdatetime.h>
#include <qcheckbox.h>
#include <qmessagebox.h>
#include <qtextedit.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qdialog.h>
#include <qcheckbox.h>
#include <qdir.h>
#include <qslider.h>
#include <qtable.h>

#include "settingscontrol.h"
#include "calendar.h"
#include "settings.h"

extern CalendarSettings config;
/*!
    \fn SettingsControl::SettingsControl(QWidget* parent)
	Constructor of class object. Sets variables and GUI elements into initial state. Constructs also SettingWidget object
	\param parent pointer to parent widget (application window).
 */
SettingsControl::SettingsControl(QWidget* parent)
 : SettingsWidget(parent)
{
//ustawienie wszystkich parametrw
//dodanie parametrw rodzaju zdarzenia
	EventType->insertItem(tr("Periodic"));
	EventType->insertItem(tr("In weekdays"));
	number->setValue(1);
	number->setMinValue(1);
	dayMon->setEnabled(FALSE);
	dayTue->setEnabled(FALSE);
	dayWed->setEnabled(FALSE);
	dayThu->setEnabled(FALSE);
	dayFri->setEnabled(FALSE);
	daySat->setEnabled(FALSE);
	daySun->setEnabled(FALSE);
	PeriodType->insertItem(tr("days"));
	PeriodType->insertItem(tr("weeks"));
	PeriodType->insertItem(tr("months"));
	PeriodType->insertItem(tr("years"));
	EditStartDate->setDate(QDate::currentDate());
	EditStartDate->setMinValue(QDate::currentDate());
	EditEndDate->setDate(QDate::currentDate());
	EditEndDate->setMinValue(QDate::currentDate());
	EventInput->clear();
	
	this->InfoWindowCancellation->setChecked(::config.closeInfoWindowAcceptItsContent);
	this->InfoWindowBehavior->setChecked(::config.closeOtherInfoWindows);
	
	UpdateEvents();
	loadThemeNames();
	
	checkMarkActive->setChecked(::config.bMarkActive);
	checkMarkSunday->setChecked(::config.bMarkSunday);
	
	sliderCurrentWidth->setValue(::config.currentCalendarWidth);
	
	spinMaxPastCards->setValue(::config.nMaxPastCards);
	spinIndividualEventsDelete->setValue(::config.maxDaysPastForIndividuals);
	
	PeriodicTable->setSorting(TRUE);
	
}
#include <iostream.h>


/*!
    \fn SettingsControl::~SettingsControl()
	Default and empty destructor of class
 */
SettingsControl::~SettingsControl()
{
}




/*!
    \fn SettingsControl::AddEvent()
	Adds newly entered periodic events into display list of periodic events (in Settings widget) and into global list of periodic events (in CalendarWidget object)
 */
void SettingsControl::AddEvent()
{
	QDate dStart = this->EditStartDate->date();
	QDate dEnd = this->EditEndDate->date();
	QString tekst = EventInput->text();
	int period=this->number->value();
	bool autodelete=this->checkDeleteAfterEnd->isChecked();
	int forgotafter=this->spinWarnDays->value();
	
	if(verifyEvent())
	{
	
		switch(EventType->currentItem())
		{
		case 0:
			switch(PeriodType->currentItem())
			{
			case 0:
				if(period==1)
					//DAILY 
					{
					((Calendar*)parent())->addPeriodicEvent(dStart, dEnd, tekst, CalEnums::DAILY, 1, NULL, autodelete, forgotafter);
					}
				else
					//PERIOD
					((Calendar*)parent())->addPeriodicEvent(dStart, dEnd, tekst, CalEnums::PERIOD, period, NULL, autodelete, forgotafter);
				break;
			case 1:
				//WEAKLY
				((Calendar*)parent())->addPeriodicEvent(dStart, dEnd, tekst, CalEnums::WEEKLY, period, NULL, autodelete, forgotafter);
				break;
			case 2:
				//MONTHLY
				((Calendar*)parent())->addPeriodicEvent(dStart, dEnd, tekst, CalEnums::MONTHLY, period, NULL, autodelete, forgotafter);
				break;
			case 3:
				//YEARLY	
				((Calendar*)parent())->addPeriodicEvent(dStart, dEnd, tekst, CalEnums::YEARLY, period, NULL, autodelete, forgotafter);
				break;
			}
			break;
		case 1:
			//day of week event
			bool days[]={dayMon->isChecked(),dayTue->isChecked(),dayWed->isChecked(),dayThu->isChecked(),dayFri->isChecked(),daySat->isChecked(),daySun->isChecked()};
			((Calendar*)parent())->addPeriodicEvent(dStart, dEnd, tekst, CalEnums::DAYS, 0, days, autodelete, forgotafter);
			break;
		}
	this->UpdateEvents();
	}
}


/*!
    \fn SettingsControl::verifyEvent()
	Validates data entered by user. Displays warnings and descriptions of mistakes. 
	\return Result of validation
 */
bool SettingsControl::verifyEvent()
{
	if(EditEndDate->date()==EditStartDate->date())
	{
		displayErrorMessage(tr("Start date and end data are the same!\nIf you really want to do this, please create individual event\nby clicking on calendar card, writing description of event in a window and accepting \nby \"OK\" key"));
		return FALSE;
	}
	if(EditEndDate->date() < EditStartDate->date())
	{
		displayErrorMessage(tr("End date is less than start date\nPlease, insert correct dates."));
		return FALSE;
	}
	if (EventInput->text().isEmpty())
	{
		displayErrorMessage(tr("Event description is empty.\nShould be filled."));
		return FALSE;
	}
	if (EventType->currentItem()==1 && !dayMon->isChecked() && !dayTue->isChecked() && !dayWed->isChecked() && !dayThu->isChecked() && !dayFri->isChecked() && !daySat->isChecked() && !daySun->isChecked())
	{
		displayErrorMessage(tr("No weekday was checked. \nCheck one of them."));
		return FALSE;
	}
	return TRUE;
}


/*!
    \fn SettingsControl::displayErrorMessage(QString tekst)
	Method displays message box with given text
	\param tekst text to display in a message box. 
*/
void SettingsControl::displayErrorMessage(QString tekst)
{
	QMessageBox msg(tr("Warning! Error!"), tr("Inserted data isn't proper. Event haven't been saved\n\nDescription:\n") + tekst, QMessageBox::Critical, QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton, QMessageBox::NoButton, this);
	msg.exec();
}


/*!
    \fn SettingsControl::UpdateEvents()
	Updates display list of events on Settings' widget
 */
void SettingsControl::UpdateEvents()
{
	this->Events->clear();
	int i=0;
	QString tekst;
	QString numer;
	CalendarEventPeriodic *temp=((Calendar*)parent())->getPeriodicEvent(i);
	//this->PeriodicTable->setNumRows(4);
	this->PeriodicTable->setNumRows(22);
	while(temp)
	{
		switch(temp->eventType)
		{
			case CalEnums::DAYS:
				tekst=tr("In weekdays:");
				for (int i=0; i<7;i++)
					{
					if(temp->days[i])
						tekst.append(" " + QDate::shortDayName(i+1));
					}
				tekst.append(". ");
				break;
			case CalEnums::PERIOD:
				tekst=tr("Periodic, every");
				numer=QString("%1").arg(temp->period);
				tekst.append(numer);
				tekst.append(" days. ");
				break;
			case CalEnums::DAILY:
				tekst=tr("Every day. ");
				break;
			case CalEnums::WEEKLY:
				if (temp->period==1)
					tekst=tr("Every week. ");
				else
				{
					tekst=tr("Every ");
					numer=QString("%1").arg(temp->period);
					tekst.append(numer);
					tekst.append(tr(" weeks. "));
				}
				break;
			case CalEnums::MONTHLY:
				if (temp->period==1)
					tekst=tr("Every month. ");
				else
				{
					tekst=tr("Every ");
					numer=QString("%1").arg(temp->period);
					tekst.append(numer);
					tekst.append(tr(" months. "));
				}
				break;
			case CalEnums::YEARLY:
				if (temp->period==1)
					tekst=tr("Every year. ");
				else
				{
					tekst=tr("Every ");
					numer=QString("%1").arg(temp->period);
					tekst.append(numer);
					tekst.append(tr(" years. "));
				}
				break;
		}
		tekst.append(temp->text);
		
		this->Events->insertItem(tekst);
		this->PeriodicTable->setText(i,4,tekst);
		
		i++;
		temp=((Calendar*)parent())->getPeriodicEvent(i);
	}
	this->Events->setSelected(0, TRUE);	
}

#include <iostream.h>

/*!
    \fn SettingsControl::updateData(int i)
	Updates edit windows which display start date and end date of a selected event
	\param i index of event
 */
void SettingsControl::updateData(int i)
{
	CalendarEventPeriodic* temp = ((Calendar*)parent())->getPeriodicEvent(i);
	this->StartDate->setText(temp->startDate.toString(Qt::LocalDate));
	this->EndDate->setText(temp->endDate.toString(Qt::LocalDate));
}




/*!
    \fn SettingsControl::accept()
	Method is called to save changes done in Settings' widget. Currently only data from first tab is saved, because the other setting don't requires acceptation.
 */
void SettingsControl::accept() 
{
	saveChanges();
	
	QDialog::accept();
}


/*!
    \fn SettingsControl::RemoveEvent()
	Removes event from display list of periodic events and from global list of periodic events. 
 */
void SettingsControl::RemoveEvent()
{
	int n=Events->index(Events->selectedItem());
	
	((Calendar*)parent())->removePeriodicEvent(n);
	this->UpdateEvents();
}


/*!
    \fn SettingsControl::saveChanges()
	Method saves changes done on settings' window. 
 */
void SettingsControl::saveChanges()
{

	((Calendar*)parent())->savePeriodicEvents();
	::config.closeInfoWindowAcceptItsContent=this->InfoWindowCancellation->isChecked();
	::config.closeOtherInfoWindows=InfoWindowBehavior->isChecked();
	::config.bMarkActive=checkMarkActive->isChecked();
	::config.bMarkSunday=checkMarkSunday->isChecked();

	::config.setPixmap(CalEnums::NO_PIXMAP, comboNoEventBitmap->currentItem());
	::config.setPixmap(CalEnums::PXD, comboDayBitmap->currentItem());
	::config.setPixmap(CalEnums::PXI, comboIndivBitmap->currentItem());
	::config.setPixmap(CalEnums::PXW, comboWeekBitmap->currentItem());
	::config.setPixmap(CalEnums::PXM, comboMonthBitmap->currentItem());
	::config.setPixmap(CalEnums::PXY, comboYearBitmap->currentItem());
	::config.setPixmap(CalEnums::PXALERT, comboAlertBitmap->currentItem());


	
	
    //change of calendarwidth
	
	//the following line does:
	//calculates difference between current position of a line and position saved in application
	//  i = current_position - position_saved
	// current position is calculated (throug the equals of ratios: current_position/current_calendar_width=saved_position/saved_calendar width):
	// current_position = current_calendar_width*saved_position/saved_calendar_width
	//current_calendar_width is calculated as following:
	//current_calendar_width = current_slider(%) * screen_width / 100
	
	
	int i = (this->sliderCurrentWidth->value()*::config.screenWidth/100)*(::config.lineposition)/(::config.kalendarzWidth) - ::config.lineposition;
	
	::config.currentCalendarWidth=this->sliderCurrentWidth->value();
	::config.kalendarzWidth=::config.currentCalendarWidth*::config.screenWidth/100;
	((Calendar*)parent())->moveLine(i);
	::config.maxDaysPastForIndividuals=spinIndividualEventsDelete->value();
	::config.nMaxPastCards=spinMaxPastCards->value();
	
	::config.setPos(((QWidget*)parent())->pos());
	::config.saveSettings();

	//reload theme
	::config.theme.name=this->comboTheme->currentText();
	((Calendar*)parent())->reloadCalendars();

	((QWidget*)parent())->repaint();
}


/*!
    \fn SettingsControl::discardChanges()
	Method discard changes done on settings' window and closes this window. It reloads list of periodic events from a file.
 */
void SettingsControl::discardChanges()
{
	//reload old value of currentWidth
	sliderCurrentWidth->setValue(::config.currentCalendarWidth);

	((Calendar*)parent())->reloadCalendars();
	
	UpdateEvents();
	close();
	
}





/*!
    \fn SettingsControl::loadThemeNames()
	Method loads theme names - it means names of directories placed in ~/.kalendarz/themes/ directory.
 */
void SettingsControl::loadThemeNames()
{
        QDir d( ::config.PATH+"/themes/", QString::null, QDir::Name | QDir::IgnoreCase, QDir::Dirs);
		int index=0;
	comboTheme->insertItem("default");
	int setDefault=-1;
    for (uint i = 0; i < d.count(); i++ )
		{
		if (d[i]!="." && d[i]!="..")
			{
        	comboTheme->insertItem(d[i]);
			index++;
			if(d[i]==::config.theme.name)
				{
				setDefault=index;
				}
			}
		}
	if(setDefault<0)
		{
			//check if it should be default theme
			if(::config.theme.name=="default" || ::config.theme.name.isEmpty())
				comboTheme->setCurrentItem(0);
			else //probably it is a custom theme name (path)
				{
					comboTheme->insertItem(::config.theme.name);
					index++;
					comboTheme->setCurrentItem(index);
				}
		}
	else
		{
			comboTheme->setCurrentItem(setDefault);
		}
}


/*!
    \fn SettingsControl::changeThemeName()
	Slot run after change of theme name in settings window. It changes a thumbnail of theme.
 */
void SettingsControl::changeThemeName()
{
	//read theme name from combo
	QString themeName=comboTheme->currentText();
	//calculate path to thubnail
	QString path=::config.PATH+"themes/"+themeName + "/mini.xpm";
	//load thumbnail to control.
	QPixmap mini;
	mini.load(path);
	pixmapTheme->setPixmap(mini);
}


/*!
    \fn SettingsControl::setPeriodic(QString text, QDate date)
	method used to put initial data about event when event is created based on existing individual event. 
	@param text initial text of events
	@param date initial start date of event
 */
void SettingsControl::setPeriodic(QString text, QDate date)
{
	EventInput->setText(text);
	EditStartDate->setDate(date);
	EditEndDate->setDate(date);
}


/*!
    \fn SettingsControl::reloadThumbnails()
	This method refresh thumbnails displayed in settings window, used to connect event type to bitmap. 
 */
void SettingsControl::reloadThumbnails()
{
	//clear all values of pixmap combos
	comboDayBitmap->clear();
	comboIndivBitmap->clear();
	comboMonthBitmap->clear();
	comboNoEventBitmap->clear();
	comboWeekBitmap->clear();
	comboYearBitmap->clear();	
	comboAlertBitmap->clear();
	
	for (int j=0; j< 7; j++)
		{
		
		//setting bitmap buttons.
			comboIndivBitmap->insertItem(*(::config.pixmaps[j]));
			comboYearBitmap->insertItem(*(::config.pixmaps[j]));
			comboMonthBitmap->insertItem(*(::config.pixmaps[j]));
			comboWeekBitmap->insertItem(*(::config.pixmaps[j]));
			comboDayBitmap->insertItem(*(::config.pixmaps[j]));
			comboNoEventBitmap->insertItem(*(::config.pixmaps[j]));
			comboAlertBitmap->insertItem(*(::config.pixmaps[j]));
		}

	comboDayBitmap->setCurrentItem(::config.getPixmapIndex(CalEnums::PXD));
	comboIndivBitmap->setCurrentItem(::config.getPixmapIndex(CalEnums::PXI));
	comboMonthBitmap->setCurrentItem(::config.getPixmapIndex(CalEnums::PXM));
	comboNoEventBitmap->setCurrentItem(::config.getPixmapIndex(CalEnums::NO_PIXMAP));
	comboWeekBitmap->setCurrentItem(::config.getPixmapIndex(CalEnums::PXW));
	comboYearBitmap->setCurrentItem(::config.getPixmapIndex(CalEnums::PXY));	
	comboAlertBitmap->setCurrentItem(::config.getPixmapIndex(CalEnums::PXALERT));


}
