/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qnamespace.h>
#include <qpoint.h>
#include <qdatetime.h>
#include <qtextedit.h>
#include <qpainter.h>
#include <qlabel.h>
#include <qfont.h>

#include "consts.h"
#include "wcard.h"
#include "wcalendar.h"

#include <iostream.h>

extern CalendarSettings config;

/*!
    \fn CalendarCardWidget::CalendarCardWidget(QWidget *parent, const char *name)
	Constructor of class object
	<br>It sets parametres into initial values and connects signals and slots
\param parent pointer to parent window 
	\param name name of widget - unused
 */
CalendarCardWidget::CalendarCardWidget(QWidget *parent, const char *name)
 : QPushButton(parent, name)
{	
	connect(this , SIGNAL( toggled(bool) ), this, SLOT( slotToggled(bool) ));
	connect(this , SIGNAL( shift(int) ), parent, SLOT( shift(int) ));
	connect(this , SIGNAL( closeInfos()), parent, SLOT( closeInfos() ));
	infoWindow=NULL;
	bFree=TRUE;
	mouse_pressed=0;
	mouse_moved=0;
	position=0;
	parentWidget=parent;
	curBitmap=/*alertBitmap=normalBitmap=*/NULL;
	pControl=NULL;
	state=CalEnums::NO_PIXMAP;
	openedInfoWindow=FALSE;
	bIsPast=FALSE;
	this->setFlat(TRUE);
}


/*!
    \fn CalendarCardWidget::~CalendarCardWidget()
	Default destructor of object. 
	It destroys InfoWidget object if eny exists.
 */
CalendarCardWidget::~CalendarCardWidget() 
{
	if (infoWindow) 
		delete infoWindow;
	pControl=NULL;
	curBitmap=NULL;
}


/*!
    \fn CalendarCardWidget::initialiseCardWidgets()
	Second part of initialisation of object.
	<br>Sets states of all cards. 
 */
void CalendarCardWidget::initialiseCardWidgets()
{
	int temp_state=state;
	this->state=8; //inpossible in normal use. Just to cause state change.
	updateState(temp_state);
}

/*!
    \fn CalendarCardWidget::cleanCardData()
	Cleans data stored in associated CalendarCardData object
 */
void CalendarCardWidget::cleanCardData()
	{
		if(pControl)
			updateState(pControl->resetCard());
	}
	
/*!
    \fn CalendarCardWidget::addIndividualEvent(QString ev)
	Adds individual event to associated CalendarCardData object
	\param ev tekst of event or empty string if event is being removed
 */
void CalendarCardWidget::addIndividualEvent(QString ev)
	{
		if(pControl)
			updateState(pControl->setIndividualEvent(ev, bIsPast));
	}
	
/*!
    \fn CalendarCardWidget::updateState(int s)
	Changes state of object into given state. Changes displayed bitmap.
	\param s new state
 */
void CalendarCardWidget::updateState(int s)
{
	if (s!=state)
	{
		state=s;
		curBitmap=::config.getPixmap(state);
		this->setPixmap( *curBitmap );
	}
}

/*!
    \fn CalendarCardWidget::slotToggled(bool on)
	Slot to handle QPushButton::toggled signal.
	<br>Displays new InfoWidget window if card was toggled on<br>
	<br>Hides InfoWidget window if card was toggled off and accepts or reject individual event which was entered into InfoWidget window
	\param on boolean value. State of button. 
 */
void CalendarCardWidget::slotToggled(bool on)
{
	QPoint temp=this->mapToGlobal(QPoint(0, ::config.HEIGHT));
	if (temp.y()>::config.screenHeight-200)
		temp=this->mapToGlobal(QPoint(0, 0-200));
	if (on && !openedInfoWindow)
	{ //opening
		openedInfoWindow=TRUE;
		//close others first
		closeInfos();
		//create window
		this->infoWindow = new InfoWidget((QWidget*)this, "Info", FALSE, Qt::WStyle_Customize+Qt::WStyle_NoBorder );
		
		
		this->infoWindow->setGeometry(temp.x(), temp.y(),150,200);
		
		this->infoWindow->Date->setText(pControl->getDate().toString()); 
		
		this->infoWindow->Individual->setText(pControl->getIndividualEventText());
		
		if(!bIsPast)
		{
			this->infoWindow->Periodical->setText(pControl->getPeriodicEventText());
		}
		else
		{
			this->infoWindow->Periodical->hide();
			
//			cout << "x = " << infoWindow->Individual->x() << "\ty = " << infoWindow->Individual->y() << "\n";
			
//			cout << "posx = " << infoWindow->Individual->pos().x() << "\tposy = " << infoWindow->Individual->pos().y()<< "\n"; 		
			this->infoWindow->Individual->setGeometry(0,infoWindow->Periodical->pos().y(), infoWindow->Individual->size().width(), infoWindow->Individual->size().height()+infoWindow->Periodical->size().height());
//			cout << "x = " << infoWindow->Individual->x() << "\ty = " << infoWindow->Individual->y() << "\n";
//			cout << "posx = " << infoWindow->Individual->pos().x() << "\tposy = " << infoWindow->Individual->pos().y() << "\n";
			//,infoWindow->Individual->x(), infoWindow->Individual->y()+infoWindow->Periodical->y());
		}
		this->infoWindow->show();
		
		connect(this->infoWindow->OK , SIGNAL( clicked() ), this, SLOT(slotAcceptedInfo() ));
		connect(this->infoWindow->Cancel , SIGNAL( clicked() ), this, SLOT(slotCancelledInfo() ));
    }
	else 
	{ //closing
		if(openedInfoWindow) //if InfoWindow is still open - card button was pressed
			{ //acceptance 
			if(::config.closeInfoWindowAcceptItsContent)
				{
				QString t1 = this->infoWindow->Individual->text();
				this->addIndividualEvent(t1);
				}
			openedInfoWindow=FALSE;
		}
		delete this->infoWindow;	
		this->infoWindow=NULL;
	}
this->repaint();
}


/*!
    \fn CalendarCardWidget::slotAcceptedInfo()
	Modifies (adds, changes or removes) individual event which was entered into InfoWidget window. 
 */
void CalendarCardWidget::slotAcceptedInfo()
{
	QString temp = this->infoWindow->Individual->text();
	this->addIndividualEvent(temp);
	openedInfoWindow=FALSE;
	this->setOn(FALSE);
}


/*!
    \fn CalendarCardWidget::slotCancelledInfo()
	Rejects changes done in InfoWidget window.
 */
void CalendarCardWidget::slotCancelledInfo()
{
	if(this->pControl->IndividualEvent)
		this->infoWindow->Individual->setText(this->pControl->IndividualEvent->text);
	openedInfoWindow=FALSE;
	this->setOn(FALSE);
}


/*!
    \fn CalendarCardWidget::paintEvent( QPaintEvent * e)
	Overloaded member of QPushButton::paintEvent. 
	<br>Method is responsible for correctly displaying of object
	\param e system event object
 */
void CalendarCardWidget::paintEvent( QPaintEvent * e)
{
	if (bFree) return;
	QPainter paint(this);

	if(openedInfoWindow && ::config.bMarkActive)	
	{
		paint.setFont(::config.theme.activeCardFont);
		paint.setPen(::config.theme.activeCardFontColor);
	}
	else if(pControl->date.dayOfWeek()==7 && ::config.bMarkSunday)
	{
		paint.setFont(::config.theme.sundayFont);
		paint.setPen(::config.theme.sundayFontColor);
	}
	else 
	{
		paint.setFont(::config.theme.cardFont);
		paint.setPen(::config.theme.cardFontColor);
	}
	
	
	if (state == CalEnums::NO_PIXMAP || pControl->date != ::config.current_date)
		paint.drawPixmap(0,0, *curBitmap);
	else
		paint.drawPixmap(0,0, *(::config.pixmaps[::config.current_pixmaps[6]]));
		
	paint.setBackgroundMode(Qt::TransparentMode);
	if (pControl)
	{
		paint.drawText( QRect(0,2,::config.WIDTH,::config.HEIGHT/3), AlignCenter, QString("%1").arg(pControl->getDate().day()));
		paint.drawText( QRect(0,::config.HEIGHT/3+1,::config.WIDTH,::config.HEIGHT/3), AlignCenter, QDate::shortDayName(pControl->getDate().dayOfWeek()));
 	}
}


/*!
    \fn CalendarCardWidget::moveCalendarCard(int pos)
	Moves the object on the screen.
	\param pos shift value
 */
int CalendarCardWidget::moveCalendarCard(int pos)
{
	this->position+=pos;
	
			move(position,0);
			if (this->infoWindow)
			{
				QPoint temp=this->mapToGlobal(QPoint(0, ::config.HEIGHT));
				this->infoWindow->move(temp);
			}
	return 0;
}

/*!
    \fn CalendarCardWidget::mousePressEvent ( QMouseEvent * e )
	Overloaded member of QPushButton::mousePressEvent.
	\param e - system event object
 */
void CalendarCardWidget::mousePressEvent ( QMouseEvent * e )
{
	//if right button was right-clicked try to add new periodic event. 
	if(e->button() == Qt::RightButton)
	((Calendar*)(parent()->parent()))->changeAddPeriodicMenu(this);
	mouse_position=e->globalX();	
	mouse_pressed=e->button();
	QPushButton::mousePressEvent ( e);
}



/*!
    \fn CalendarCardWidget::mouseReleaseEvent(QMouseEvent * e)
	Overloaded member of QPushButton::mouseReleaseEvent
	\param e system event object
 */
void CalendarCardWidget::mouseReleaseEvent(QMouseEvent * e)
{
	mouse_pressed=0;
	if (!mouse_moved)
	{
		QPushButton::mouseReleaseEvent( e);
	}
	mouse_moved=0;
}


/*!
    \fn CalendarCardWidget::mouseMoveEvent(QMouseEvent * e)
	Overloaded member of QWidget::mouseMoveEvent
	\param e system event object
 */
void CalendarCardWidget::mouseMoveEvent(QMouseEvent * e)
{
	if (mouse_pressed==Qt::LeftButton && (mouse_moved || abs(e->globalX()-mouse_position)>5))
		{
			mouse_moved=1;
			shift(e->globalX()-mouse_position);
			mouse_position=e->globalX();
		}	
	QWidget::mouseMoveEvent(e);
}

/*!
    \fn CalendarCardWidget::freeInfoWidget()
	Accepts opened InfoWidget if there is any
 */
void CalendarCardWidget::freeInfoWidget()
{
	if (this->infoWindow)
		this->slotAcceptedInfo();
}


/*!
    \fn CalendarCardWidget::initialiseCardData(long position, CalendarCardData* control, long numControl)
	Associates CalendarCardData object to this object
 	\param position horizontal position on the screen
	\param control pointer to CalendarCardData object, which will be associated with this object
	\param numControl index of associated CalendarCardData object in the global list of CalendarCardData objects
 */
void CalendarCardWidget::initialiseCardData(long position, CalendarCardData* control, long numControl)
{
    if (!control)
		{
//			cout << "WARNING, CalendarCardData= NULL\n";
			return;
		}
	this->freeAssociation(FALSE);
	this->associateCardData(control);
	this->updateState(this->pControl->getState());
	this->numControl=numControl;
	this->position=position;	
	this->moveCalendarCard(0);
}


/*!
    \fn CalendarCardWidget::isInfoNull()
	Returns if there is InfoWidget Window opened for object.
	\return \em TRUE if there is InfoWindow created for this object \n Otherwise \em FALSE
 */
bool CalendarCardWidget::isInfoNull()
{
	if(infoWindow==NULL)
		return TRUE;
	else 
		return FALSE;
}


/*!
    \fn CalendarCardWidget::associateCardData(CalendarCardData * control) 
	Associates this object to CalendarCardData object
	\param control pointer to CalendarCardData which is to be associated to this object
 */
void CalendarCardWidget::associateCardData(CalendarCardData * control) 
{
	this->pControl=control; 
	this->pControl->pWidget=this; 
}
