/***************************************************************************
 *   Copyright (C) 2004 by Adam Skowronski                                 *
 *   tytus@oldfield.org.pl                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef WCARD_H
#define WCARD_H


#include <qpushbutton.h>
#include <qpixmap.h>
#include <qevent.h>
#include <qbutton.h>

#include "carddata.h"
#include "eindiv.h"
#include "InfoWidget.h"

#include "settings.h"


class CalendarCardData;

/**
Class is responsible for displaying one card of the calendar as a part of GUI.
<br>Class inherits from QPushButton mainly because QPushButton has <i>toggle</i> slot implemented and can display bitmap on it. 
@brief Calendar card class
@author Adam Skowronski
*/
class CalendarCardWidget : public QPushButton {
Q_OBJECT
public:
    CalendarCardWidget(QWidget *parent = 0, const char *name = 0);
    ~CalendarCardWidget();
	
    void initialiseCardWidgets();
    void initialiseCardData(long position, CalendarCardData* control, long numControl);
    void associateCardData(CalendarCardData * control);
    void cleanCardData();
    /*! \fn freeAssociation(bool bfree) 
Frees association of CalendarCardData object. */
    void freeAssociation(bool bfree){ this->bFree=bfree;if(bFree)pControl=NULL;  }

    void addIndividualEvent(QString ev);
    void updateState(int s);

	int moveCalendarCard(int pos);

	/** \name Event's handlers */
	/** @{ */
    void paintEvent( QPaintEvent * e);
    void mousePressEvent ( QMouseEvent * e );
    void mouseReleaseEvent(QMouseEvent * e);
    void mouseMoveEvent(QMouseEvent * e);
	/** @} */

	void freeInfoWidget();
    bool isInfoNull();

    
		
	
	CalendarCardData *pControl;	/**< pointer to associated CalendarCardData object */
	int state;					/**< state of object. It is CalEnums::PIXMAP_STATES converted to int. Should be the same as a state of associated CalendarCardData object */
	QPixmap *curBitmap;			/**< pointer to currently displayed bitmap */
	bool bFree;					/**< Indicates if there is any CalendarCardData object associated to this object */
    int numControl;				/**< index of CalendarCardData object associated to this object */
    long position;				/**< horizontal position on the screen. Can be from \c -WIDTH to \c screen_width \c + \c WIDTH  */
    bool bIsPast;				/**< indicates that card is inside past calendar */

    InfoWidget* infoWindow;		/**< pointer to widget in which events are displayed */

public slots:
    void slotToggled(bool on);   
    void slotAcceptedInfo();
    void slotCancelledInfo();
protected:
	
    int mouse_position;			/**< position of mouse when button was pressed - used for calcutating of a shift of whole calendar */
    int mouse_pressed;			/**< indicates if mouse button was pressed - only if mouse button is pressed position of the mouse is calculated */
	int mouse_moved;			/**< indicates if movement of the mouse can cause movement of the calendar - if mouse is pressed and a current position of mouse is further than 5 from mouse_position then mouse_moved is set to true */ 
    QWidget* parentWidget;		/**< pointer to parent widget (CalendarWidget) */
	bool openedInfoWindow;		/**< indicates that InfoWindow is opened. Because info window could be closed in two manners: *
		- By clicking a card which cause signal togglebutton and this signal is handled and contents of InfoWindow are saved if it is needed. 
		- by OK button (or Cancel as well) which cause saving of InfoWindow and calling togglebutton signal which will cause a second time saving of info window contents. <br>This variable prevents from double saving */
    
signals:
    void shift(int i);			/**< signal emitted to parent window to shift whole calendar */
    void closeInfos();			/**< signal emitted to parent window to close all InfoWidget windows if configuration says so*/
    void changeAddPeriodicSubmenu(QString text);
								/**< signal emitted to parent window to change menu in AddPeriodic submenu */
};

#endif
